@@ -129,6 +129,7 @@ enum {
     PROP_LAST
 };
 
+static gboolean accessibility_invoke (GnomeProgram *program, gboolean init);
 static void gnome_program_finalize      (GObject           *object);
 
 static GQuark quark_get_prop = 0;
@@ -716,6 +717,8 @@ gnome_program_finalize (GObject* object)
 		g_array_free (self->_priv->top_options_table, TRUE);
 	self->_priv->top_options_table = NULL;
 
+	g_slist_free (self->_priv->accessibility_modules);
+
 	GNOME_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
 }
 
@@ -1657,6 +1660,8 @@ gnome_program_parse_args (GnomeProgram *program)
 	}
 }
 
+#ifndef G_OS_WIN32 /* No GNOME a11y stuff is implemented for real on Windows anyway yet. */
+
 static char *
 find_accessibility_module (GnomeProgram *program, const char *libname)
 {
@@ -1757,10 +1762,12 @@ accessibility_invoke (GnomeProgram *program, gboolean init)
 	return TRUE;
 }
 
+#endif
 
 static void
 accessibility_init (GnomeProgram *program)
 {
+#ifndef G_OS_WIN32
 	int i;
 	gboolean do_init;
 	const char *env_var;
@@ -1786,13 +1793,16 @@ accessibility_init (GnomeProgram *program)
 
 	if ((env_var = g_getenv (GNOME_ACCESSIBILITY_ENV)))
 		do_init = atoi (env_var);
-	else
+	else {
+		GConfClient* gc = gconf_client_get_default ();
 		do_init = gconf_client_get_bool (
-			gconf_client_get_default (),
-			GNOME_ACCESSIBILITY_KEY, NULL);
+			gc, GNOME_ACCESSIBILITY_KEY, NULL);
+		g_object_unref (gc);
+	}
 
 	if (do_init)
 		accessibility_invoke (program, TRUE);
+#endif
 }
 
 /**
