@@ -29,7 +29,7 @@
  * Gnome utility routines.
  * (C)  1997, 1998, 1999 the Free Software Foundation.
  *
- * Author: Miguel de Icaza, 
+ * Author: Miguel de Icaza,
  */
 #include <config.h>
 
@@ -46,11 +46,16 @@
 #include <string.h>
 #include <unistd.h>
 #include <glib.h>
-#include <sys/stat.h>
+#ifndef G_OS_WIN32
 #include <pwd.h>
+#endif
 #include <limits.h>
-#include <libgnome/gnome-program.h>
-#include <libgnome/gnome-util.h>
+#include "gnome-program.h"
+#include "gnome-util.h"
+
+#ifdef G_OS_WIN32
+#include <windows.h>
+#endif
 
 #if defined(__APPLE__) && defined(HAVE_NSGETENVIRON) && defined(HAVE_CRT_EXTERNS_H)
 # include <crt_externs.h>
@@ -67,17 +72,18 @@
 char *
 gnome_util_user_shell (void)
 {
+#ifndef G_OS_WIN32
 	struct passwd *pw;
 	int i;
 	const char *shell;
-	static char *shells [] = {
+	const char shells [][14] = {
 		/* Note that on some systems shells can also
 		 * be installed in /usr/bin */
 		"/bin/bash", "/usr/bin/bash",
 		"/bin/zsh", "/usr/bin/zsh",
 		"/bin/tcsh", "/usr/bin/tcsh",
 		"/bin/ksh", "/usr/bin/ksh",
-		"/bin/csh", "/bin/sh", NULL
+		"/bin/csh", "/bin/sh"
 	};
 
 	if (geteuid () == getuid () &&
@@ -96,7 +102,7 @@ gnome_util_user_shell (void)
 		}
 	}
 
-	for (i = 0; shells [i]; i++) {
+	for (i = 0; i != G_N_ELEMENTS (shells); i++) {
 		if (access (shells [i], X_OK) == 0) {
 			return g_strdup (shells[i]);
 		}
@@ -107,6 +113,20 @@ gnome_util_user_shell (void)
 
 	/* Placate compiler.  */
 	return NULL;
+#else
+	/* g_find_program_in_path() always looks also in the Windows
+	 * and System32 directories, so it should always find either cmd.exe
+	 * or command.com.
+	 */
+	char *retval = g_find_program_in_path ("cmd.exe");
+
+	if (retval == NULL)
+		retval = g_find_program_in_path ("command.com");
+
+	g_assert (retval != NULL);
+
+	return retval;
+#endif
 }
 
 /**
@@ -124,7 +144,7 @@ const char *
 g_extension_pointer (const char * path)
 {
 	char * s, * t;
-	
+
 	g_return_val_if_fail(path != NULL, NULL);
 
 	/* get the dot in the last element of the path */
@@ -133,7 +153,7 @@ g_extension_pointer (const char * path)
 		s = strrchr(t, '.');
 	else
 		s = strrchr(path, '.');
-	
+
 	if (s == NULL)
 		return path + strlen(path); /* There is no extension. */
 	else {
@@ -148,7 +168,7 @@ g_extension_pointer (const char * path)
  * @value: The value to assign to the environment variable.
  * @overwrite: If %TRUE, overwrite the existing @name variable in the
  * environment.
- * 
+ *
  * Adds "@name=@value" to the environment. Note that on systems without setenv,
  * this leaks memory so please do not use inside a loop or anything like that.
  * The semantics are the same as the glibc setenv() (if setenv() exists, it is
@@ -158,7 +178,7 @@ g_extension_pointer (const char * path)
  * then %0 is returned and the value is not changed.
  *
  * Returns: %0 on success, %-1 on error
- * 
+ *
  **/
 int
 gnome_setenv (const char *name, const char *value, gboolean overwrite)
@@ -167,11 +187,11 @@ gnome_setenv (const char *name, const char *value, gboolean overwrite)
 	return setenv (name, value != NULL ? value : "", overwrite);
 #else
 	char *string;
-	
+
 	if (! overwrite && g_getenv (name) != NULL) {
 		return 0;
 	}
-	
+
 	/* This results in a leak when you overwrite existing
 	 * settings. It would be fairly easy to fix this by keeping
 	 * our own parallel array or hash table.
@@ -184,12 +204,12 @@ gnome_setenv (const char *name, const char *value, gboolean overwrite)
 /**
  * gnome_unsetenv:
  * @name: The environment variable to unset.
- * 
+ *
  * Description: Removes @name from the environment.
  * In case there is no native implementation of unsetenv,
  * this could cause leaks depending on the implementation of
  * environment.
- * 
+ *
  **/
 void
 gnome_unsetenv (const char *name)
@@ -201,7 +221,7 @@ gnome_unsetenv (const char *name)
 	int i, len;
 
 	len = strlen (name);
-	
+
 	/* Mess directly with the environ array.
 	 * This seems to be the only portable way to do this.
 	 */
@@ -220,12 +240,12 @@ gnome_unsetenv (const char *name)
 
 /**
  * gnome_clearenv:
- * 
+ *
  * Description: Clears out the environment completely.
  * In case there is no native implementation of clearenv,
  * this could cause leaks depending on the implementation
  * of environment.
- * 
+ *
  **/
 void
 gnome_clearenv (void)
@@ -238,7 +258,6 @@ gnome_clearenv (void)
 #endif
 }
 
-
 /* Deprecated: */
 /**
  * gnome_is_program_in_path:
@@ -246,7 +265,7 @@ gnome_clearenv (void)
  *
  * Deprecated, use #g_find_program_in_path
  *
- * Returns: %NULL if program is not on the path or a string 
+ * Returns: %NULL if program is not on the path or a string
  * allocated with g_malloc() with the full path name of the program
  * found.
  */
